/*
 *  MODULE   : smail.cpp
 *
 *  PURPOSE  : Simple Mail class implementation derived form CSMapi
 *
 *  FUNCTIONS:
 *
 *
 * Copyright 1994 WinSoft Corporation. All rights reserved.
 */

#define STRICT
//-------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------
#include <windows.h>      // Windows definitions
#include <stdio.h>        // Used with remove function  
#include <alloc.h>        // Used with farfree function
#include <mem.h>          // Used with memset function
#include <string.h>       // string functions
//#include "resource.h"     // Application resource ids
#include "smail.h"        // Simple mail definitions

//-------------------------------------------------------------------------
// Defines
//-------------------------------------------------------------------------
#define MESSAGE_UNREAD  0 // Message in the listbox has not been read yet
#define MESSAGE_READ    1 // Message in the listbox was read already

//-------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------
extern "C" {
BOOL WINAPI _export ReadMailDlgProc(HWND, UINT, WPARAM, LPARAM);
}

//-------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------
CSimpleMail :: CSimpleMail( HWND hParent )
             : CSimpleMAPI( hParent ), nMaxMailMsgs(128)
{
  // Create the E-Mail message array. This array holds upto nMaxMailMsgs
  // MAILMSG structures
  pMsgs     = new MAILMSG[nMaxMailMsgs];
  memset( pMsgs, 0, sizeof(MAILMSG) * nMaxMailMsgs );
  nMsgCount = 0;
}

//-------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------
CSimpleMail :: ~CSimpleMail()
{
  // Loop through the email message array, check that there is a valid
  // MAILMSG structure, delete the message handle, and delete any files
  // associated with that message.
  for (int i = 0; i < nMaxMailMsgs; i++ )
  {
    if (pMsgs[i].hMsg) DeleteMsgHandle(pMsgs[i].hMsg);
    if (pMsgs[i].lpFiles)
    {
      farfree(pMsgs[i].lpFiles[i].lpszPathName);
      farfree(pMsgs[i].lpFiles[i].lpszFileName);
      delete [] pMsgs[i].lpFiles;
    }
  }
  // Delete the message array
  delete [] pMsgs;
}

//-------------------------------------------------------------------------
// ListMessages
//-------------------------------------------------------------------------
void CSimpleMail :: ListEMail( HWND hListbox )
{
  // Delete if any, all e-mail message information currently in the
  // e-mail message array since we are baou to reload it.
  DeleteAllMsgInfo();

  // Call the CSimpleMAPI class method to list all the message available 
  // in our inbox. This method will call a method called ReceiveMsgHandle.
  // This method is defined in the parent class as abstract so we must
  //define it somewhere in our implementation of CSimpleMAIL.
  ListAllMsgs();
  
  // Now we have received all message handles from the parent class. Loop
  // through the array of messages and call the CSimpleMAPI ReadMail to
  // get the subject of the message we have in our inbox. For now all we
  // want to do is display the subject. 
  lpMapiMessage lpMsg;
  SendMessage( hListbox, WM_SETREDRAW, 0, 0 ); 
  for (int i = 0; i < nMaxMailMsgs; i++ )
  {
    if (!pMsgs[i].hMsg) continue;
    lpMsg = ReadMail( pMsgs[i].hMsg, MAPI_ENVELOPE_ONLY | MAPI_PEEK );
    ULONG ulError = ErrorCode();
    if (ulError != SMAPI_SUCCESS )
    {
      char szMsg[255];
      ErrorString( ulError, szMsg, sizeof(szMsg));
      MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
      break;
    }
    // Here we add the subject string to the listbox
    UINT nIndex = (UINT)SendMessage( hListbox, 
                                     LB_ADDSTRING, 
                                     0, (LPARAM)lpMsg->lpszSubject );

    // Keep the index into the message array as well as the read status
    // of the message in the item data DWORD in the listbox
    SendMessage( hListbox, LB_SETITEMDATA, nIndex, MAKELPARAM(i,MESSAGE_UNREAD) ); 
    // Delete the message structure that MAPI created for us
    DelMapiMsgStruct( lpMsg );
  }
  SendMessage( hListbox, WM_SETREDRAW, 1, 0 ); 
  InvalidateRect( hListbox, NULL, TRUE );
}

//-------------------------------------------------------------------------
// ListUnreadMessages
//-------------------------------------------------------------------------
void CSimpleMail :: ListUnreadEMail( HWND hListbox )
{
  // Delete if any, all e-mail message information currently in the
  // e-mail message array since we are baou to reload it.
  DeleteAllMsgInfo();

  // Call the CSimpleMAPI class method to list all the unread messages
  // available in our inbox. 
  // This method will call a method called ReceiveMsgHandle
  // which is defined in the parent class as abstract so we must
  // define it somewhere in our implementation of CSimpleMAIL.
  ListUnreadMsgs();
  
  // Now we have received all message handles from the parent class. Loop
  // through the array of messages and call the CSimpleMAPI ReadMail to
  // get the subject of the message we have in our inbox. For now all we
  // want to do is display the subject. 
  lpMapiMessage lpMsg;
  SendMessage( hListbox, WM_SETREDRAW, 0, 0 ); 
  for (int i = 0; i < nMaxMailMsgs; i++ )
  {
    if (!pMsgs[i].hMsg) continue;
    lpMsg = ReadMail( pMsgs[i].hMsg, MAPI_ENVELOPE_ONLY );
    ULONG ulError = ErrorCode();
    if (ulError != SMAPI_SUCCESS )
    {
      char szMsg[255];
      ErrorString( ulError, szMsg, sizeof(szMsg));
      MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
      break;
    }
    // Here we add the subject string to the listbox
    UINT nIndex = (UINT)SendMessage( hListbox, 
                                     LB_ADDSTRING, 
                                     0, (LPARAM)lpMsg->lpszSubject );
    // Keep the index into the message array as well as the read status
    // of the message in the item data DWORD in the listbox
    SendMessage( hListbox, LB_SETITEMDATA, nIndex, MAKELPARAM(i,MESSAGE_UNREAD) ); 
    // Delete the message structure that MAPI created for us
    DelMapiMsgStruct( lpMsg );
  }
  SendMessage( hListbox, WM_SETREDRAW, 1, 0 ); 
  InvalidateRect( hListbox, NULL, TRUE );
}

//-------------------------------------------------------------------------
// ReadEMail
//-------------------------------------------------------------------------
void CSimpleMail :: ReadEMail( HWND hListbox )
{
  // Get the listbox current selection
  UINT nLBIndex = (UINT)SendMessage( hListbox, LB_GETCURSEL, 0, 0L );
  // From the current selection, obtain the index into our message array
  UINT nMsgIndex = LOWORD(SendMessage(hListbox, LB_GETITEMDATA, nLBIndex, 0L ));

  // Call the CSimpleMAPI ReadMail called with the handle retrieved from
  // the message array using the retrived index
  lpMapiMessage lpMsg = ReadMail( pMsgs[nMsgIndex].hMsg, NULL);
  
  ULONG ulError = ErrorCode();
  if ( ulError != SMAPI_SUCCESS )
  {
    char szMsg[255];
    ErrorString( ulError, szMsg, sizeof(szMsg) );
    MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
  }
  else
  {
    // Copy all the file attachment descriptions. They will be copied to
    // the disk when we read the E-Mail. We must be able to delete them 
    // when the user deletes them.
    if (lpMsg->nFileCount)
    {
      pMsgs[nMsgIndex].nFileCount  = lpMsg->nFileCount;
      pMsgs[nMsgIndex].lpFiles     = new MapiFileDesc[(UINT)lpMsg->nFileCount];
      for ( int i = 0; i < lpMsg->nFileCount; i++ )
      {
        pMsgs[nMsgIndex].lpFiles[i].flFlags       = lpMsg->lpFiles[i].flFlags;
        pMsgs[nMsgIndex].lpFiles[i].nPosition     = lpMsg->lpFiles[i].nPosition;
        pMsgs[nMsgIndex].lpFiles[i].lpszPathName  = strdup(lpMsg->lpFiles[i].lpszPathName);
        pMsgs[nMsgIndex].lpFiles[i].lpszFileName  = strdup(lpMsg->lpFiles[i].lpszFileName);
      }
    }
    // Display a dialog box that contacins all the information about the
    // E-Mail we jsut read.
    HINSTANCE hInstance = (HINSTANCE)GetWindowWord( m_hWndParent, GWW_HINSTANCE );
    FARPROC lpfnProc = MakeProcInstance( (FARPROC)ReadMailDlgProc, hInstance );
    DialogBoxParam( hInstance, "ReadMailDlg", m_hWndParent, 
                    (DLGPROC)lpfnProc, (LPARAM)lpMsg );
    FreeProcInstance((FARPROC)lpfnProc);

    // Delete the message structure that MAPI created for us
    DelMapiMsgStruct(lpMsg);
    // Change the status of the message to READ since we have accessed it.
    SendMessage(hListbox, LB_SETITEMDATA, nLBIndex, MAKELPARAM(nMsgIndex,MESSAGE_READ) );
  }  
}  

//-------------------------------------------------------------------------
// DeleteEMail
//-------------------------------------------------------------------------
void CSimpleMail :: DeleteEMail( HWND hListbox )
{
  // Get the index for the current selection on the listbox
  UINT nLBIndex = (UINT)SendMessage( hListbox, LB_GETCURSEL, 0, 0L );

  // Get the item data DWORD from the selection
  LRESULT lData = SendMessage(hListbox, LB_GETITEMDATA, nLBIndex, 0L );

  // Extract the message array index and the message read flag
  UINT nMsgIndex = LOWORD(lData);
  UINT fMsgRead  = HIWORD(lData);

  // Call the CSimpleMAPI DeleteMail method
  DeleteMail( pMsgs[nMsgIndex].hMsg );
  
  // If there was an error display it
  ULONG ulError = ErrorCode();
  if ( ulError != SMAPI_SUCCESS )
  {
    char szMsg[255];
    ErrorString( ulError, szMsg, sizeof(szMsg) );
    MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
  }
  else
  {
    // Delete all file attachments if any exist for this message
    if (fMsgRead)
    {
      for (int i = 0; i < pMsgs[nMsgIndex].nFileCount; i++)
      {
        if (remove(pMsgs[nMsgIndex].lpFiles[i].lpszPathName) != 0)
        {
          MessageBox( m_hWndParent, 
                      "Error deleting attachments to this message.", 
                      "Mail Error", MB_OK|MB_ICONSTOP );
        }
        farfree(pMsgs[nMsgIndex].lpFiles[i].lpszPathName);
        farfree(pMsgs[nMsgIndex].lpFiles[i].lpszFileName);
      }
    }
    // Delete the message handle 
    DeleteMsgHandle( pMsgs[nMsgIndex].hMsg );

    // Clear out the slot within the message array
    pMsgs[nMsgIndex].hMsg       = 0;
    pMsgs[nMsgIndex].nFileCount = 0;

    // Delete the memory allocated for the file attachments
    delete [] pMsgs[nMsgIndex].lpFiles;
    pMsgs[nMsgIndex].lpFiles    = 0;

    // Delete the string from the listbox
    SendMessage( hListbox, LB_DELETESTRING, nLBIndex, 0L );
  }  
}  

//-------------------------------------------------------------------------
// SaveEMail
//-------------------------------------------------------------------------
void CSimpleMail :: SaveEMail( HWND hListbox )
{
  // Get the index for the current selection on the listbox
  UINT nLBIndex = (UINT)SendMessage( hListbox, LB_GETCURSEL, 0, 0L );

  // Get the message array index from the item data for the selection
  UINT nMsgIndex = LOWORD(SendMessage(hListbox, LB_GETITEMDATA, nLBIndex, 0L ));
  
  // Call the ReadMail CSimpleMAPI method. We call this function first 
  // because we must have a fully filled MapiMessage structure. It is
  // filled by calling the ReadMail method. After we fill the structure
  // with the message information, we can then save it.
  lpMapiMessage lpMsg = ReadMail( pMsgs[nMsgIndex].hMsg, NULL);
  
  // If there was an error display it
  ULONG ulError = ErrorCode();
  if ( ulError != SMAPI_SUCCESS )
  {
    char szMsg[255];
    ErrorString( ulError, szMsg, sizeof(szMsg) );
    MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
  }
  else
  {
    // Clear the previous file descriptors if there were any.
    if ( pMsgs[nMsgIndex].nFileCount ) 
    {
      for (int i = 0; i < pMsgs[nMsgIndex].nFileCount; i++)
      {
        farfree(pMsgs[nMsgIndex].lpFiles[i].lpszPathName);
        farfree(pMsgs[nMsgIndex].lpFiles[i].lpszFileName);
      }
      delete [] pMsgs[nMsgIndex].lpFiles;
      pMsgs[nMsgIndex].nFileCount = 0;
      pMsgs[nMsgIndex].lpFiles    = 0;
    }

    // Copy all the file attachment descriptions. They will be copied to
    // the disk when we read the E-Mail. We must be able to delete them 
    // when the user deletes the mail message.
    if (lpMsg->nFileCount)
    {
      pMsgs[nMsgIndex].nFileCount  = lpMsg->nFileCount;
      pMsgs[nMsgIndex].lpFiles     = new MapiFileDesc[(UINT)lpMsg->nFileCount];
      for ( int i = 0; i < lpMsg->nFileCount; i++ )
        pMsgs[nMsgIndex].lpFiles[i] = lpMsg->lpFiles[i];
    }
    // Call the CSimpleMAPI method SaveMail.
    HMSG hNewHandle = SaveMail( pMsgs[nMsgIndex].hMsg, lpMsg);

    // If there was an error display it
    if ( ErrorCode()!= SMAPI_SUCCESS )
    {
      char szMsg[255];
      ErrorString( ulError, szMsg, sizeof(szMsg) );
      MessageBox( m_hWndParent, szMsg, "Mail Error", MB_OK|MB_ICONSTOP );
    }
    else
    {
      // Copy the new handle given by SaveMail into the message array slot.
      pMsgs[nMsgIndex].hMsg = hNewHandle;

      // Delete the structure we created by calling ReadMail
      DelMapiMsgStruct(lpMsg);

      // Set the message state to READ instead of UNREAD
      SendMessage(hListbox, LB_SETITEMDATA, nLBIndex, MAKELPARAM(nMsgIndex,MESSAGE_READ) );
    }
  }  
}  

//-------------------------------------------------------------------------
// ReceiveMsgHandle
//-------------------------------------------------------------------------
void CSimpleMail :: ReceiveMsgHandle( HMSG hMsg )
{
  // Loop through the message array looking for 
  // a slot to save the message handle.
  for (int i = 0; i < nMaxMailMsgs; i++ )
  {
    if (pMsgs[i].hMsg) continue;
    else
    {
      pMsgs[i].hMsg = hMsg;
      nMsgCount++;
      break;
    }  
  }  
}
  
//-------------------------------------------------------------------------
// DeleteAllMsgInfo
//-------------------------------------------------------------------------
void CSimpleMail :: DeleteAllMsgInfo( void )
{
  // Loop through the message array deleting any valid
  // message slots
  for (int i = 0; i < nMaxMailMsgs; i++ )
  {
    if (!pMsgs[i].hMsg) continue;
    else
    {
      DeleteMsgHandle(pMsgs[i].hMsg);
      if (pMsgs[i].nFileCount) delete pMsgs[i].lpFiles;
    }
  }  
  // Clear the array of messages with zeroes
  memset( pMsgs, 0, sizeof(MAILMSG) * nMaxMailMsgs );
}


//-------------------------------------------------------------------------
// ReadMailDlgProc
//-------------------------------------------------------------------------
BOOL WINAPI _export ReadMailDlgProc( HWND   hWnd, 
                                     UINT   wMessage, 
                                     WPARAM wParam, 
                                     LPARAM lParam )
{
/*
  switch(wMessage)
  {
	 case WM_INITDIALOG:
	 {
		// lParam contains a pointer to a lpMapiMessage
		lpMapiMessage lpMsg = (lpMapiMessage) lParam;

		// Get control handles
		HWND hSubject     = GetDlgItem( hWnd, IDS_SUBJECT );
		HWND hOriginator  = GetDlgItem( hWnd, IDS_ORIGINATOR );
		HWND hMsg         = GetDlgItem( hWnd, IDE_MSG );
		HWND hRecipients  = GetDlgItem( hWnd, IDE_RECIPIENTS );
		HWND hDate        = GetDlgItem( hWnd, IDS_DATE );
		HWND hAttach      = GetDlgItem (hWnd, IDL_ATTACHMENTS );
		// Set each control with information within the
		// MapiMessage structure
		SetWindowText( hSubject,    lpMsg->lpszSubject );
		SetWindowText( hOriginator, lpMsg->lpOriginator->lpszName);
		SetWindowText( hMsg,        lpMsg->lpszNoteText );
		SetWindowText( hDate,       lpMsg->lpszDateReceived);

		// Build the recipient list
		LPSTR lpsz = new char[1024];
		memset( lpsz, 0, 1024 );
		if (lpMsg->nRecipCount)
		{
		  for (int i = 0; i < lpMsg->nRecipCount; i++ )
		  {
			 lstrcat( lpsz, lpMsg->lpRecips[i].lpszName );
			 lstrcat( lpsz, ";" );
		  }
		}
		SetWindowText( hRecipients, lpsz);

		// Build the attachments list
		memset( lpsz, 0, 1024 );
		if (lpMsg->nFileCount)
		{
		  for (int i = 0; i < lpMsg->nFileCount; i++ )
		  {
			 lstrcat( lpsz, lpMsg->lpFiles[i].lpszFileName );
			 SendMessage( hAttach, LB_ADDSTRING, 0, (LPARAM)lpsz );
		  }
		}

		delete lpsz;
	 }
	 return TRUE;

	 case WM_COMMAND:
	 {
		switch( wParam )
		{
		  case IDOK:
			 EndDialog( hWnd, IDOK );
			 return TRUE;

		  default:
			 return FALSE;
		}
	 }

	 default:
		return FALSE;
  }
*/
}


void CSimpleMail :: SendMail( char *subject, char *noteText, char *recipient )
{
  if (m_lhSession == NULL)
  {
	 m_ulError = SMAPI_E_INVALID_SESSION;
	 return;
  }

  MapiMessage Msg;
  memset( &Msg, 0, sizeof(MapiMessage) );
  MapiRecipDesc Recipient;
  memset( &Recipient, 0, sizeof(MapiRecipDesc) );
  Recipient.ulRecipClass = MAPI_TO;
  Recipient.lpszName = recipient;

  Msg.lpszSubject = subject;
  Msg.lpszNoteText = noteText;
  Msg.nRecipCount = 1;
  Msg.lpRecips = &Recipient;

  m_ulError = m_lpfnMAPISendMail( m_lhSession,
											 (ULONG)m_hWndParent,
											 &Msg, MAPI_DIALOG, 0 );
}
