/*
 *  MODULE   s: mail.h
 *
 *  PURPOSE  : Simple Mail class definition derived from CSimpleMAPI
 *
 * Copyright 1994 WinSoft Corporation. All rights reserved.
 */

#ifndef MAIL_H
#define MAIL_H

//-------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------
#include "smapi.h"      // Simple MAPI class definition

//-------------------------------------------------------------------------
// Simple MAIL class
//-------------------------------------------------------------------------
class CSimpleMail : public CSimpleMAPI
{
  public:
    // Constructor/Destructor
    CSimpleMail( HWND hParent );
    virtual ~CSimpleMail();

	 void SendMail( char *subject = NULL, char *noteText = NULL,
	                                             char *recipient = NULL );

	 // Methods that list all or unread E-mails. The message handles
    // are sent to a method called ReceiveMsgHandle. See below.
    virtual void ListEMail( HWND hListbox );
    virtual void ListUnreadEMail( HWND hListbox );

    // Receiver of message handles. This functions receives from the
    // parent class a listing of all message handles. It is 
    // the responsibility of this class to store these handles for later
    // use.
    virtual void ReceiveMsgHandle( HMSG hMsg ); 

    // Method to Read an E-Mail message
    virtual void ReadEMail( HWND hListbox );

    // Method to Delete an E-Mail message
    virtual void DeleteEMail( HWND hListbox );

    // Method to save and E-mail message
    virtual void SaveEMail( HWND hListbox );

    // Method to Send an Email message
    virtual void SendEMail( void ) { SendMail(); }
    
  protected:
    // Delete all E-Mail information that is currently stored in the
    // pMsgs array.
    void DeleteAllMsgInfo( void );

    // Structure that holds all pertinent information about an E-Mail
    // message. This class builds an array of these structures
    typedef struct { HMSG           hMsg; 
                     ULONG          nFileCount;
                     lpMapiFileDesc lpFiles;
                   } MAILMSG;
                   
    const UINT  nMaxMailMsgs;   // Maximum number of messages allowed
    UINT        nMsgCount;      // Current number of messages
    MAILMSG     *pMsgs;         // E-Mail message array
};

#endif
