/*
 *  MODULE   : smapi.h
 *
 *  PURPOSE  : Simple MAPI class definition
 *
 * Copyright 1994 WinSoft Corporation. All rights reserved.
 */

#ifndef SMAPI_H
#define SMAPI_H

//-------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------
#include "mapi.h"     // Simple MAPI definition file supplied by Microsoft

//-------------------------------------------------------------------------
// Simple MAPI class error messages
//-------------------------------------------------------------------------
#define SMAPI_SUCCESS                    SUCCESS_SUCCESS          
#define SMAPI_USER_ABORT                 MAPI_USER_ABORT               
#define SMAPI_E_FAILURE                  MAPI_E_FAILURE                
#define SMAPI_E_LOGIN_FAILURE            MAPI_E_LOGIN_FAILURE          
#define SMAPI_E_DISK_FULL                MAPI_E_DISK_FULL              
#define SMAPI_E_INSUFFICIENT_MEMORY      MAPI_E_INSUFFICIENT_MEMORY    
#define SMAPI_E_ACCESS_DENIED            MAPI_E_ACCESS_DENIED    
#define SMAPI_E_TOO_MANY_SESSIONS        MAPI_E_TOO_MANY_SESSIONS      
#define SMAPI_E_TOO_MANY_FILES           MAPI_E_TOO_MANY_FILES         
#define SMAPI_E_TOO_MANY_RECIPIENTS      MAPI_E_TOO_MANY_RECIPIENTS    
#define SMAPI_E_ATTACHMENT_NOT_FOUND     MAPI_E_ATTACHMENT_NOT_FOUND   
#define SMAPI_E_ATTACHMENT_OPEN_FAILURE  MAPI_E_ATTACHMENT_OPEN_FAILURE
#define SMAPI_E_ATTACHMENT_WRITE_FAILURE MAPI_E_ATTACHMENT_WRITE_FAILURE         
#define SMAPI_E_UNKNOWN_RECIPIENT        MAPI_E_UNKNOWN_RECIPIENT                
#define SMAPI_E_BAD_RECIPTYPE            MAPI_E_BAD_RECIPTYPE                    
#define SMAPI_E_NO_MESSAGES              MAPI_E_NO_MESSAGES                      
#define SMAPI_E_INVALID_MESSAGE          MAPI_E_INVALID_MESSAGE                  
#define SMAPI_E_TEXT_TOO_LARGE           MAPI_E_TEXT_TOO_LARGE                   
#define SMAPI_E_INVALID_SESSION          MAPI_E_INVALID_SESSION    
#define SMAPI_E_TYPE_NOT_SUPPORTED       MAPI_E_TYPE_NOT_SUPPORTED 
#define SMAPI_E_AMBIGUOUS_RECIPIENT      MAPI_E_AMBIGUOUS_RECIPIENT
#define SMAPI_E_MESSAGE_IN_USE           MAPI_E_MESSAGE_IN_USE   
#define SMAPI_E_NETWORK_FAILURE          MAPI_E_NETWORK_FAILURE  
#define SMAPI_E_INVALID_EDITFIELDS       MAPI_E_INVALID_EDITFIELDS
#define SMAPI_E_INVALID_RECIPS           MAPI_E_INVALID_RECIPS  
#define SMAPI_E_NOT_SUPPORTED            MAPI_E_NOT_SUPPORTED  

// These messages are not part of the MAPI implementation. They
// were created to describe error conditions specific to the class
#define SMAPI_E_DLL_NOT_FOUND           40
#define SMAPI_E_MAPI_DLL_FUNC_NOT_FOUND 41
#define SMAPI_E_DELETE_MSG_HANDLE       42 
#define SMAPI_E_MAPI_NOT_PRESENT        43
#define SMAPI_E_INVALID_MSG_HANDLE      44

//-------------------------------------------------------------------------
// Typedefs
//-------------------------------------------------------------------------
typedef ATOM HMSG;    // Handle to message ID.

//-------------------------------------------------------------------------
// Simple MAPI class
//-------------------------------------------------------------------------
class CSimpleMAPI
{
  public:
    // Constructor and Destructor
    CSimpleMAPI( HWND hParent );
    virtual ~CSimpleMAPI();

    // Mail Logon/Logoff methods
    virtual void Logon( LPSTR lpszUserId = NULL, LPSTR lpszPassword = NULL );
    virtual void Logoff( void );

    // Methods that list all messages and receive the msg handles
    virtual void ListAllMsgs( void );
    virtual void ListUnreadMsgs( void );
    virtual void DeleteMsgHandle( HMSG hMsg );
    virtual void ReceiveMsgHandle( HMSG hMsg ) = 0;

    // Mail reading related methods
    virtual lpMapiMessage ReadMail( HMSG hMsg, ULONG ulFlags );
    virtual void          DelMapiMsgStruct( lpMapiMessage );

    // Mail deletion methods
    virtual void DeleteMail( HMSG hMsg );

    // Mail SendDocuments methods
    // This method is used to implement the Send... menu in MAPI
    // compliant applications
    virtual void SendDocuments( LPSTR lpszDelimChar  = NULL, 
                                LPSTR lpszFilesPath  = NULL,
                                LPSTR lpszFileNames  = NULL );

    // MAPI address details method
    virtual void AddressDetails( lpMapiRecipDesc lpRecip );

    // MAPI name resolution methods
    virtual void ResolveName( LPSTR lpszName, lpMapiRecipDesc lppRecip );

    // Save message methods
    virtual HMSG SaveMail( HMSG hMsg, lpMapiMessage lpMsg );

    // Send message methods
    virtual void SendMail( void );

    // Error code methods
    virtual ULONG ErrorCode( void ) { return m_ulError; }
    virtual BOOL  ErrorString( ULONG ulErrorCode, LPSTR lpszBuffer, UINT nSize );
    
  protected:
    HWND        m_hWndParent;     // Handle to the parent window
    HINSTANCE   m_hMAPIDLL;       // Handle to the MAPI.DLL instance
    ULONG       m_ulError;        // Error code for the last MAPI transaction
    LHANDLE     m_lhSession;      // Handle to MAPI session

//  private:
    // Typedefs that define the prototype for each of the
    // MAPI calls inside the MAPI dll. We use these typedefs to 
    // declare variables of each of these types and assign the
    // value returned by GetProcAdress to them.
    typedef ULONG (WINAPI * TMAPILogon)( ULONG ulUIParam, 
                                         LPSTR lpszName, 
                                         LPSTR lpszPassword,
                                         FLAGS flFlags, 
                                         ULONG ulReserved,
                                         LPLHANDLE lplhSession );
 
    typedef ULONG (WINAPI * TMAPILogoff)( LHANDLE lhSession, 
                                          ULONG ulUIParam, 
                                          FLAGS flFlags,
                                          ULONG ulReserved );

    typedef ULONG (WINAPI * TMAPISendMail)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            lpMapiMessage lpMessage, 
                                            FLAGS flFlags,
                                            ULONG ulReserved);

    typedef ULONG (WINAPI * TMAPISendDocuments)( ULONG ulUIParam, 
                                                  LPSTR lpszDelimChar,
                                                  LPSTR lpszFilePaths, 
                                                  LPSTR lpszFileNames,
                                                  ULONG ulReserved );

    typedef ULONG (WINAPI * TMAPIFindNext)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            LPSTR lpszMessageType, 
                                            LPSTR lpszSeedMessageID,
                                            FLAGS flFlags, 
                                            ULONG ulReserved,
                                            LPSTR lpszMessageID );

    typedef ULONG (WINAPI * TMAPIReadMail)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            LPSTR lpszMessageID, 
                                            FLAGS flFlags,
                                            ULONG ulReserved, 
                                            lpMapiMessage FAR *lppMessageOut);

    typedef ULONG (WINAPI * TMAPISaveMail)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            lpMapiMessage pMessage, 
                                            FLAGS flFlags,
                                            ULONG ulReserved, 
                                            LPSTR lpszMessageID );

    typedef ULONG (WINAPI * TMAPIDeleteMail)( LHANDLE lhSession, 
                                              ULONG ulUIParam,
                                              LPSTR lpszMessageID, 
                                              FLAGS flFlags,
                                              ULONG ulReserved );

    typedef ULONG (WINAPI * TMAPIFreeBuffer)( LPVOID pv );
              
    typedef ULONG (WINAPI * TMAPIAddress)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            LPSTR plszCaption, 
                                            ULONG nEditFields,
                                            LPSTR lpszLabels, 
                                            ULONG nRecips,
                                            lpMapiRecipDesc lpRecips, 
                                            FLAGS flFlags, 
                                            ULONG ulReserved, 
                                            LPULONG lpnNewRecips, 
                                            lpMapiRecipDesc FAR *lppNewRecips);

    typedef ULONG (WINAPI * TMAPIDetails)( LHANDLE lhSession, 
                                            ULONG ulUIParam,
                                            lpMapiRecipDesc lpRecip, 
                                            FLAGS flFlags, 
                                            ULONG ulReserved);
         
    typedef ULONG (WINAPI * TMAPIResolveName)( LHANDLE lhSession, 
                                                ULONG ulUIParam,
                                                LPSTR lpszName, 
                                                FLAGS flFlags,
                                                ULONG ulReserved, 
                                                lpMapiRecipDesc FAR *lppRecip);

    // These private data members hold the procedure address of each of the
    // MAPI functions inside MAPI.DLL 
    TMAPILogon          m_lpfnMAPILogon;
    TMAPILogoff         m_lpfnMAPILogoff;
    TMAPISendMail       m_lpfnMAPISendMail;
    TMAPISendDocuments  m_lpfnMAPISendDocuments;
    TMAPIFindNext       m_lpfnMAPIFindNext;
    TMAPIReadMail       m_lpfnMAPIReadMail;
    TMAPISaveMail       m_lpfnMAPISaveMail;
    TMAPIDeleteMail     m_lpfnMAPIDeleteMail;
    TMAPIFreeBuffer     m_lpfnMAPIFreeBuffer;
    TMAPIAddress        m_lpfnMAPIAddress;
    TMAPIDetails        m_lpfnMAPIDetails;
    TMAPIResolveName    m_lpfnMAPIResolveName;
};

#endif
